import json
import requests
from python_graphql_client import GraphqlClient

from .login import get_bearer_auth_base, get_bearer_authorization_header_value


QUERY = 'query'
VARIABLES = 'variables'
METHOD = 'method'
LOG_PREFIX = 'logPrefix'
PAYLOAD = 'payload'
AUTHORIZE = 'authorize'
EXTRA_HEADERS = 'extraHeaders'
USE_DEFAULT_HEADERS = 'useDefaultHeaders'


def retrieve_graphql(url):
    def with_config(config):
        use_auth = config.get(AUTHORIZE, False)
        auth = get_bearer_auth_base() if use_auth else None
        client = GraphqlClient(endpoint=url, auth=auth)
        prefix_message = config.get(LOG_PREFIX, 'Perform a POST request')
        prefix = f"{prefix_message} to {url}"
        print(prefix, 'variables', config[VARIABLES])
        response = client.execute(config[QUERY], config[VARIABLES])
        print(prefix, 'response', response)
        return response
    return with_config


def retrieve_http(url):
    def with_config(config):
        method = config.get(METHOD, requests.get)
        prefix_message = config.get(LOG_PREFIX, f"Perform a {method.__name__} request")
        prefix = f"{prefix_message} to {url}"

        def get_default_headers():
            headers = {}
            if config.get(USE_DEFAULT_HEADERS, True):
                headers.update({
                    'Content-Type': 'application/json'
                })
                if config.get(AUTHORIZE, False):
                    headers['Authorization'] = get_bearer_authorization_header_value()
            headers.update(config.get(EXTRA_HEADERS, {}))
            print(prefix, 'headers', headers)
            return headers

        payload = config.get(PAYLOAD, None)
        print(prefix, 'payload', payload)
        response = method(
            url=url,
            data=None if payload is None else json.dumps(payload),
            headers=get_default_headers(),
        )
        print(prefix, 'response', response)
        data = response.json()
        print(prefix, 'JSON response', data)
        return data
    return with_config



import re

import pytest

from .config import get_store_reader

get_env = get_store_reader()


def get_marker(request):
    def with_value(value):
        marker = request.node.get_closest_marker(value)
        if marker is None:
            return None
        return marker.args[0]
    return with_value


def get_driver_params(env_var):
    def with_default_values(default_values):
        requested_drivers = get_env(env_var)
        if requested_drivers is not None:
            return requested_drivers.split(',')
        return default_values
    return with_default_values


def get_driver_name(request):
    drivers_to_skip = get_marker(request)('skip_drivers') or []
    for driver_to_skip in drivers_to_skip:
        name = driver_to_skip.get('name', None)
        reason = driver_to_skip.get('reason', 'skipped for this driver: {}'.format(driver_to_skip))
        if name == request.param:
            pytest.skip(reason)
            break
    return request.param


def select_driver(driver_name):
    def with_drivers(drivers):
        matched_driver = None
        for driver in drivers:
            if re.match(rf"driver.+{driver_name}", driver.__name__) is not None:
                if matched_driver is not None:
                    raise Exception(f"More than one driver matches <{driver_name}>: <{matched_driver.__name__}>, and <{driver.__name__}>")
                matched_driver = driver
        if matched_driver is not None:
            return matched_driver
        raise Exception(f"Could not find driver with name <{driver_name}>" + \
            ' in the list of drivers:\n' + \
            '\n'.join(map(lambda d: d.__name__, drivers)) + \
            '\n')
    return with_drivers



from . import cache
from . import config
from . import date_diff
from . import driver
from . import json_web_token
from . import login
from . import memoize
from . import prop
from . import retrieve
from . import web_sockets
